@@ -1,32 +1,43 @@
-{-# LANGUAGE CPP #-}
-{-# LANGUAGE TypeFamilies, FlexibleContexts, ConstraintKinds #-}
+{-# LANGUAGE DataKinds #-}
+{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TypeOperators #-}
 
 module Database.Persist.Class.PersistUnique
-  (PersistUniqueRead(..)
-  ,PersistUniqueWrite(..)
-  ,getByValue
-  ,insertBy
-  ,insertUniqueEntity
-  ,replaceUnique
-  ,checkUnique
-  ,onlyUnique
-  ,defaultPutMany
-  ,persistUniqueKeyValues
+  ( PersistUniqueRead(..)
+  , PersistUniqueWrite(..)
+  , OnlyOneUniqueKey(..)
+  , onlyOneUniqueDef
+  , AtLeastOneUniqueKey(..)
+  , atLeastOneUniqueDef
+  , NoUniqueKeysError
+  , MultipleUniqueKeysError
+  , getByValue
+  , getByValueUniques
+  , insertBy
+  , insertUniqueEntity
+  , replaceUnique
+  , checkUnique
+  , onlyUnique
+  , defaultPutMany
+  , persistUniqueKeyValues
   )
   where
 
-import Database.Persist.Types
-import Control.Exception (throwIO)
 import Control.Monad (liftM)
-import Control.Monad.IO.Class (liftIO, MonadIO)
-import Data.List ((\\), deleteFirstsBy, nubBy)
-import Data.Function (on)
+import Control.Monad.IO.Class (MonadIO)
 import Control.Monad.Trans.Reader (ReaderT)
+import Data.Function (on)
+import Data.List ((\\), deleteFirstsBy)
+import Data.List.NonEmpty (NonEmpty(..))
+import qualified Data.List.NonEmpty as NEL
+import qualified Data.Map as Map
+import Data.Maybe (catMaybes)
+import Data.Text (Text)
+import GHC.TypeLits (ErrorMessage(..))
+
+import Database.Persist.Types
 import Database.Persist.Class.PersistStore
 import Database.Persist.Class.PersistEntity
-import Data.Monoid (mappend)
-import Data.Text (unpack, Text)
-import Data.Maybe (catMaybes)
 
 -- | Queries against 'Unique' keys (other than the id 'Key').
 --
@@ -98,6 +109,7 @@ class (PersistUniqueRead backend, PersistStoreWrite backend) =>
     deleteBy
         :: (MonadIO m, PersistRecordBackend record backend)
         => Unique record -> ReaderT backend m ()
+
     -- | Like 'insert', but returns 'Nothing' when the record
     -- couldn't be inserted because of a uniqueness constraint.
     --
@@ -127,13 +139,12 @@ class (PersistUniqueRead backend, PersistStoreWrite backend) =>
         case conflict of
             Nothing -> Just `liftM` insert datum
             Just _ -> return Nothing
+
     -- | Update based on a uniqueness constraint or insert:
     --
     -- * insert the new record if it does not exist;
     -- * If the record exists (matched via it's uniqueness constraint), then update the existing record with the parameters which is passed on as list to the function.
     --
-    -- Throws an exception if there is more than 1 uniqueness constraint.
-    --
     -- === __Example usage__
     --
     -- First, we try to explain 'upsert' using <#schema-persist-unique-1 schema-1> and <#dataset-persist-unique-1 dataset-1>.
@@ -154,7 +165,7 @@ class (PersistUniqueRead backend, PersistStoreWrite backend) =>
     -- > +-----+-----+--------+
     --
     -- > upsertX :: MonadIO m => [Update User] -> ReaderT SqlBackend m (Maybe (Entity User))
-    -- > upsertX updates = upsert (User "X" 999) upadtes
+    -- > upsertX updates = upsert (User "X" 999) updates
     --
     -- > mXEnt <- upsertX [UserAge +=. 15]
     --
@@ -175,15 +186,21 @@ class (PersistUniqueRead backend, PersistStoreWrite backend) =>
     --
     -- > mSpjEnt <- upsertSpj [UserAge +=. 15]
     --
-    -- Then, it throws an error message something like "Expected only one unique key, got"
+    -- This fails with a compile-time type error alerting us to the fact
+    -- that this record has multiple unique keys, and suggests that we look for
+    -- 'upsertBy' to select the unique key we want.
     upsert
-        :: (MonadIO m, PersistRecordBackend record backend)
-        => record          -- ^ new record to insert
-        -> [Update record]  -- ^ updates to perform if the record already exists
-        -> ReaderT backend m (Entity record) -- ^ the record in the database after the operation
+        :: (MonadIO m, PersistRecordBackend record backend, OnlyOneUniqueKey record)
+        => record
+        -- ^ new record to insert
+        -> [Update record]
+        -- ^ updates to perform if the record already exists
+        -> ReaderT backend m (Entity record)
+        -- ^ the record in the database after the operation
     upsert record updates = do
         uniqueKey <- onlyUnique record
         upsertBy uniqueKey record updates
+
     -- | Update based on a given uniqueness constraint or insert:
     --
     -- * insert the new record if it does not exist;
@@ -241,10 +258,14 @@ class (PersistUniqueRead backend, PersistStoreWrite backend) =>
     -- > +-----+-----+-----+
     upsertBy
         :: (MonadIO m, PersistRecordBackend record backend)
-        => Unique record   -- ^ uniqueness constraint to find by
-        -> record          -- ^ new record to insert
-        -> [Update record] -- ^ updates to perform if the record already exists
-        -> ReaderT backend m (Entity record) -- ^ the record in the database after the operation
+        => Unique record
+        -- ^ uniqueness constraint to find by
+        -> record
+        -- ^ new record to insert
+        -> [Update record]
+        -- ^ updates to perform if the record already exists
+        -> ReaderT backend m (Entity record)
+        -- ^ the record in the database after the operation
     upsertBy uniqueKey record updates = do
         mrecord <- getBy uniqueKey
         maybe (insertEntity record) (`updateGetEntity` updates) mrecord
@@ -256,13 +277,90 @@ class (PersistUniqueRead backend, PersistStoreWrite backend) =>
     --
     -- * insert new records that do not exist (or violate any unique constraints)
     -- * replace existing records (matching any unique constraint)
+    --
     -- @since 2.8.1
     putMany
-        :: (MonadIO m, PersistRecordBackend record backend)
-        => [record]             -- ^ A list of the records you want to insert or replace.
+        ::
+        ( MonadIO m
+        , PersistRecordBackend record backend
+        )
+        => [record]
+        -- ^ A list of the records you want to insert or replace.
         -> ReaderT backend m ()
     putMany = defaultPutMany
 
+-- | This class is used to ensure that 'upsert' is only called on records
+-- that have a single 'Unique' key. The quasiquoter automatically generates
+-- working instances for appropriate records, and generates 'TypeError'
+-- instances for records that have 0 or multiple unique keys.
+--
+-- @since 2.10.0
+class PersistEntity record => OnlyOneUniqueKey record where
+    onlyUniqueP :: record -> Unique record
+
+-- | Given a proxy for a 'PersistEntity' record, this returns the sole
+-- 'UniqueDef' for that entity.
+--
+-- @since 2.10.0
+onlyOneUniqueDef
+    :: (OnlyOneUniqueKey record, Monad proxy)
+    => proxy record
+    -> UniqueDef
+onlyOneUniqueDef prxy =
+    case entityUniques (entityDef prxy) of
+        [uniq] -> uniq
+        _ -> error "impossible due to OnlyOneUniqueKey constraint"
+
+-- | This is an error message. It is used when writing instances of
+-- 'OnlyOneUniqueKey' for an entity that has no unique keys.
+--
+-- @since 2.10.0
+type NoUniqueKeysError ty =
+    'Text "The entity "
+        ':<>: 'ShowType ty
+        ':<>: 'Text " does not have any unique keys."
+    ':$$: 'Text "The function you are trying to call requires a unique key "
+        ':<>: 'Text "to be defined on the entity."
+
+-- | This is an error message. It is used when an entity has multiple
+-- unique keys, and the function expects a single unique key.
+--
+-- @since 2.10.0
+type MultipleUniqueKeysError ty =
+    'Text "The entity "
+        ':<>: 'ShowType ty
+        ':<>: 'Text " has multiple unique keys."
+    ':$$: 'Text "The function you are trying to call requires only a single "
+        ':<>: 'Text "unique key."
+    ':$$: 'Text "There is probably a variant of the function with 'By' "
+        ':<>: 'Text "appended that will allow you to select a unique key "
+        ':<>: 'Text "for the operation."
+
+-- | This class is used to ensure that functions requring at least one
+-- unique key are not called with records that have 0 unique keys. The
+-- quasiquoter automatically writes working instances for appropriate
+-- entities, and generates 'TypeError' instances for records that have
+-- 0 unique keys.
+--
+-- @since 2.10.0
+class PersistEntity record => AtLeastOneUniqueKey record where
+    requireUniquesP :: record -> NonEmpty (Unique record)
+
+-- | Given a proxy for a record that has an instance of
+-- 'AtLeastOneUniqueKey', this returns a 'NonEmpty' list of the
+-- 'UniqueDef's for that entity.
+--
+-- @since 2.10.0
+atLeastOneUniqueDef
+    :: (AtLeastOneUniqueKey record, Monad proxy)
+    => proxy record
+    -> NonEmpty UniqueDef
+atLeastOneUniqueDef prxy =
+    case entityUniques (entityDef prxy) of
+        (x:xs) -> x :| xs
+        _ ->
+            error "impossible due to AtLeastOneUniqueKey record constraint"
+
 -- | Insert a value, checking for conflicts with any unique constraints.  If a
 -- duplicate exists in the database, it is returned as 'Left'. Otherwise, the
 -- new 'Key is returned as 'Right'.
@@ -278,9 +376,12 @@ class (PersistUniqueRead backend, PersistStoreWrite backend) =>
 --
 -- First three lines return 'Left' because there're duplicates in given record's uniqueness constraints. While the last line returns a new key as 'Right'.
 insertBy
-    :: (MonadIO m
-       ,PersistUniqueWrite backend
-       ,PersistRecordBackend record backend)
+    ::
+    ( MonadIO m
+    , PersistUniqueWrite backend
+    , PersistRecordBackend record backend
+    , AtLeastOneUniqueKey record
+    )
     => record -> ReaderT backend m (Either (Entity record) (Key record))
 insertBy val = do
     res <- getByValue val
@@ -288,17 +389,6 @@ insertBy val = do
         Nothing -> Right `liftM` insert val
         Just z -> return $ Left z
 
--- | Insert a value, checking for conflicts with any unique constraints. If a
--- duplicate exists in the database, it is left untouched. The key of the
--- existing or new entry is returned
-_insertOrGet :: (MonadIO m, PersistUniqueWrite backend, PersistRecordBackend record backend)
-            => record -> ReaderT backend m (Key record)
-_insertOrGet val = do
-    res <- getByValue val
-    case res of
-        Nothing -> insert val
-        Just (Entity key _) -> return key
-
 -- | Like 'insertEntity', but returns 'Nothing' when the record
 -- couldn't be inserted because of a uniqueness constraint.
 --
@@ -351,26 +441,18 @@ insertUniqueEntity datum =
 --
 -- > mSimonConst <- onlySimonConst
 --
--- @mSimonConst@ would be Simon's uniqueness constraint. Note that @onlyUnique@ doesn't work if there're more than two constraints.
+-- @mSimonConst@ would be Simon's uniqueness constraint. Note that
+-- @onlyUnique@ doesn't work if there're more than two constraints. It will
+-- fail with a type error instead.
 onlyUnique
-    :: (MonadIO m
-       ,PersistUniqueWrite backend
-       ,PersistRecordBackend record backend)
+    ::
+    ( MonadIO m
+    , PersistUniqueWrite backend
+    , PersistRecordBackend record backend
+    , OnlyOneUniqueKey record
+    )
     => record -> ReaderT backend m (Unique record)
-onlyUnique record =
-    case onlyUniqueEither record of
-        Right u -> return u
-        Left us ->
-            requireUniques record us >>=
-            liftIO . throwIO . OnlyUniqueException . show . length
-
-onlyUniqueEither
-    :: (PersistEntity record)
-    => record -> Either [Unique record] (Unique record)
-onlyUniqueEither record =
-    case persistUniqueKeys record of
-        [u] -> Right u
-        us -> Left us
+onlyUnique = pure . onlyUniqueP
 
 -- | A modification of 'getBy', which takes the 'PersistEntity' itself instead
 -- of a 'Unique' record. Returns a record matching /one/ of the unique keys. This
@@ -394,12 +476,35 @@ onlyUniqueEither record =
 -- > |  1 | SPJ  |  40 |
 -- > +----+------+-----+
 getByValue
-    :: (MonadIO m
-       ,PersistUniqueRead backend
-       ,PersistRecordBackend record backend)
+    :: forall record m backend.
+    ( MonadIO m
+    , PersistUniqueRead backend
+    , PersistRecordBackend record backend
+    , AtLeastOneUniqueKey record
+    )
     => record -> ReaderT backend m (Maybe (Entity record))
-getByValue record =
-    checkUniques =<< requireUniques record (persistUniqueKeys record)
+getByValue record = do
+    let uniqs = requireUniquesP record
+    getByValueUniques (NEL.toList uniqs)
+
+-- | Retrieve a record from the database using the given unique keys. It
+-- will attempt to find a matching record for each 'Unique' in the list,
+-- and returns the first one that has a match.
+--
+-- Returns 'Nothing' if you provide an empty list ('[]') or if no value
+-- matches in the database.
+--
+-- @since 2.10.0
+getByValueUniques
+    ::
+    ( MonadIO m
+    , PersistUniqueRead backend
+    , PersistRecordBackend record backend
+    )
+    => [Unique record]
+    -> ReaderT backend m (Maybe (Entity record))
+getByValueUniques uniqs =
+    checkUniques uniqs
   where
     checkUniques [] = return Nothing
     checkUniques (x:xs) = do
@@ -408,21 +513,6 @@ getByValue record =
             Nothing -> checkUniques xs
             Just z -> return $ Just z
 
-requireUniques
-    :: (MonadIO m, PersistEntity record)
-    => record -> [Unique record] -> m [Unique record]
-requireUniques record [] = liftIO $ throwIO $ userError errorMsg
-  where
-    errorMsg = "getByValue: " `Data.Monoid.mappend` unpack (recordName record) `mappend` " does not have any Unique"
-
-requireUniques _ xs = return xs
-
--- TODO: expose this to users
-recordName
-    :: (PersistEntity record)
-    => record -> Text
-recordName = unHaskellName . entityHaskell . entityDef . Just
-
 -- | Attempt to replace the record of the given key with the given new record.
 -- First query the unique fields to make sure the replacement maintains
 -- uniqueness constraints.
@@ -432,11 +522,10 @@ recordName = unHaskellName . entityHaskell . entityDef . Just
 --
 -- @since 1.2.2.0
 replaceUnique
-    :: (MonadIO m
-       ,Eq record
-       ,Eq (Unique record)
-       ,PersistRecordBackend record backend
-       ,PersistUniqueWrite backend)
+    :: ( MonadIO m
+       , Eq (Unique record)
+       , PersistRecordBackend record backend
+       , PersistUniqueWrite backend )
     => Key record -> record -> ReaderT backend m (Maybe (Unique record))
 replaceUnique key datumNew = getJust key >>= replaceOriginal
   where
@@ -476,7 +565,6 @@ checkUnique = checkUniqueKeys . persistUniqueKeys
 
 checkUniqueKeys
     :: (MonadIO m
-       ,PersistEntity record
        ,PersistUniqueRead backend
        ,PersistRecordBackend record backend)
     => [Unique record] -> ReaderT backend m (Maybe (Unique record))
@@ -501,17 +589,23 @@ defaultPutMany
     => [record]
     -> ReaderT backend m ()
 defaultPutMany []   = return ()
-defaultPutMany rsD  = do
-    let uKeys = persistUniqueKeys . head $ rsD
-    case uKeys of
+defaultPutMany rsD@(e:_)  = do
+    case persistUniqueKeys e of
         [] -> insertMany_ rsD
         _ -> go
   where
     go = do
-        let rs = nubBy ((==) `on` persistUniqueKeyValues) (reverse rsD)
+        -- deduplicate the list of records in Haskell by unique key. The
+        -- previous implementation used Data.List.nubBy which is O(n^2)
+        -- complexity.
+        let rs = map snd
+                . Map.toList
+                . Map.fromList
+                . map (\r -> (persistUniqueKeyValues r, r))
+                $ rsD
 
         -- lookup record(s) by their unique key
-        mEsOld <- mapM getByValue rs
+        mEsOld <- mapM (getByValueUniques . persistUniqueKeys) rs
 
         -- find pre-existing entities and corresponding (incoming) records
         let merge (Just x) y = Just (x, y)
@@ -534,7 +628,8 @@ defaultPutMany rsD  = do
         -- replace existing records
         mapM_ (uncurry replace) krs
 
--- | The _essence_ of a unique record.
--- useful for comaparing records in haskell land for uniqueness equality.
+-- | This function returns a list of 'PersistValue' that correspond to the
+-- 'Unique' keys on that record. This is useful for comparing two @record@s
+-- for equality only on the basis of their 'Unique' keys.
 persistUniqueKeyValues :: PersistEntity record => record -> [PersistValue]
-persistUniqueKeyValues r = concat $ map persistUniqueToValues $ persistUniqueKeys r
\ No newline at end of file
+persistUniqueKeyValues = concatMap persistUniqueToValues . persistUniqueKeys
