@@ -1,7 +1,243 @@
-{-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE BangPatterns, CPP #-}
+{-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE PatternGuards #-}
+{-# LANGUAGE StandaloneDeriving, UndecidableInstances #-}
 {-# LANGUAGE ViewPatterns #-}
+
+{-|
+This module defines the Persistent entity syntax used in the quasiquoter to generate persistent entities.
+
+The basic structure of the syntax looks like this:
+
+> TableName
+>     fieldName      FieldType
+>     otherField     String
+>     nullableField  Int       Maybe
+
+You start an entity definition with the table name, in this case, @TableName@. It's followed by a list of fields on the entity, which have the basic form @fieldName FieldType@. You can indicate that a field is nullable with 'Maybe' at the end of the type.
+
+@persistent@ automatically generates an ID column for you, if you don't specify one, so the above table definition corresponds to the following SQL:
+
+> CREATE TABLE table_name (
+>     id                SERIAL PRIMARY KEY,
+>     field_name        field_type NOT NULL,
+>     other_field       varchar    NOT NULL,
+>     nullable_field    int NULL
+> );
+
+Note that the exact SQL that is generated can be customized using the 'PersistSettings' that are passed to the 'parse' function.
+
+It generates a Haskell datatype with the following form:
+
+@
+data TableName = TableName
+    { tableNameFieldName :: FieldType
+    , tableNameOtherField :: String
+    , tableNameNullableField :: Maybe Int
+    }
+@
+
+As with the SQL generated, the specifics of this are customizable.
+See the @persistent-template@ package for details.
+
+= Deriving
+
+You can add a deriving clause to a table, and the generated Haskell type will have a deriving clause with that.
+Unlike normal Haskell syntax, you don't need parentheses or commas to separate the classes, and you can even have multiple deriving clauses.
+
+> User
+>     name String
+>     age  Int
+>     deriving Eq Show
+>     deriving Ord
+
+= Unique Keys
+
+You can define a uniqueness key on a table with the following format:
+
+> User
+>    name String
+>    age  Int
+>
+>    UniqueUserName name
+
+This will put a unique index on the @user@ table and the @name@ field.
+
+= Setting defaults
+
+You can use a @default=${sql expression}@ clause to set a default for a field.
+The thing following the `=` is interpreted as SQL that will be put directly into the table definition.
+
+@
+User
+    name    Text
+    admin   Bool default=false
+@
+
+This creates a SQL definition like this:
+
+> CREATE TABLE user (
+>   id      SERIAL PRIMARY KEY,
+>   name    VARCHAR NOT NULL,
+>   admin   BOOL DEFAULT=false
+> );
+
+A restriction here is that you still need to provide a value when performing an `insert`, because the generated Haskell type has the form:
+
+@
+data User = User
+    { userName :: Text
+    , userAdmin :: Bool
+    }
+@
+
+You can work around this by using a 'Maybe Bool' and supplying 'Nothing' by default.
+
+= Custom ID column
+
+If you don't want to use the default ID column type of 'Int64', you can set a custom type with an @Id@ field.
+This @User@ has a @Text@ ID.
+
+> User
+>     Id   Text
+>     name Text
+>     age  Int
+
+If you do this, it's a good idea to set a default for the ID.
+Otherwise, you will need to use 'insertKey' instead of 'insert' when performing inserts.
+
+@
+'insertKey' (UserKey "Hello world!") (User "Bob" 32)
+@
+
+If you attempt to do @'insert' (User "Bob" 32)@, then you will receive a runtime error because the SQL database doesn't know how to make an ID for you anymore.
+So instead just use a default expression, like this:
+
+@
+User
+    Id      Text default=generate_user_id()
+    name    Text
+    age     Int
+@
+
+= Custom Primary Keys
+
+Sometimes you don't want to have an ID column, and you want a different sort of primary key.
+This is a table that stores unique email addresses, and the email is the primary key.
+We store the first and second part (eg @first\@second@) separately.
+
+@
+Email
+    firstPart   Text
+    secondPart  Text
+
+    Primary firstPart secondPart
+@
+
+This creates a table with the following form:
+
+@
+CREATE TABLE email (
+    first_part  varchar,
+    second_part varchar,
+
+    PRIMARY KEY (first_part, second_part)
+@
+
+You can specify 1 or more columns in the primary key.
+
+= Overriding SQL
+
+You can use a @sql=custom@ annotation to provide some customization on the entity and field.
+For example, you might prefer to name a table differently than what @persistent@ will do by default.
+You may also prefer to name a field differently.
+
+@
+User sql=big_user_table
+    fullName    String sql=name
+    age         Int
+@
+
+This will alter the generated SQL to be:
+
+@
+CREATE TABEL big_user_table (
+    id      SERIAL PRIMARY KEY,
+    name    VARCHAR,
+    age     INT
+);
+@
+
+= Attributes
+
+The QuasiQuoter allows you to provide arbitrary attributes to an entity or field.
+This can be used to extend the code in ways that the library hasn't anticipated.
+If you use this feature, we'd definitely appreciate hearing about it and potentially supporting your use case directly!
+
+@
+User !funny
+    field   String  !sad
+    good    Dog     !sogood
+@
+
+We can see the attributes using the 'entityAttrs' field and the 'fieldAttrs' field.
+
+@
+userAttrs = do
+    let userDefinition = 'entityDef' ('Proxy' :: 'Proxy' User)
+    let userAttributes = 'entityAttrs' userDefinition
+    let fieldAttributes = 'map' 'fieldAttrs' ('entityFields' userDefinition)
+    print userAttributes
+-- ["funny"]
+    print fieldAttributes
+-- [["sad"],["sogood"]]
+@
+
+= Documentation Comments
+
+The quasiquoter supports ordinary comments with @--@ and @#@.
+Since @persistent-2.10.5.1@, it also supports documentation comments.
+The grammar for documentation comments is similar to Haskell's Haddock syntax, with a few restrictions:
+
+1. Only the @-- | @ form is allowed.
+2. You must put a space before and after the @|@ pipe character.
+3. The comment must be indented at the same level as the entity or field it documents.
+
+An example of the field documentation is:
+
+@
+-- | I am a doc comment for a User. Users are important
+-- | to the application, and should be treasured.
+User
+    -- | Users have names. Call them by names.
+    name String
+    -- | A user can be old, or young, and we care about
+    -- | this for some reason.
+    age Int
+@
+
+The documentation is present on the `entityComments` field on the `EntityDef` for the entity:
+
+@
+>>> let userDefinition = entityDef (Proxy :: Proxy User)
+>>> entityComments userDefinition
+"I am a doc comment for a User. Users are important\nto the application, and should be treasured.\n"
+@
+
+Likewise, the field documentation is present in the `fieldComments` field on the `FieldDef` present in the `EntityDef`:
+
+@
+>>> let userFields = entityFields userDefinition
+>>> let comments = map fieldComments userFields
+>>> mapM_ putStrLn comments
+"Users have names. Call them by names."
+"A user can be old, or young, and we care about\nthis for some reason."
+@
+
+Unfortunately, we can't use this to create Haddocks for you, because <https://gitlab.haskell.org/ghc/ghc/issues/5467 Template Haskell does not support Haddock yet>.
+`persistent` backends *can* use this to generate SQL @COMMENT@s, which are useful for a database perspective, and you can use the <https://hackage.haskell.org/package/persistent-documentation @persistent-documentation@> library to render a Markdown document of the entity definitions.
+
+-}
 module Database.Persist.Quasi
     ( parse
     , PersistSettings (..)
@@ -10,22 +246,32 @@ module Database.Persist.Quasi
     , nullable
 #if TEST
     , Token (..)
+    , Line' (..)
+    , preparse
     , tokenize
     , parseFieldType
+    , empty
+    , removeSpaces
+    , associateLines
+    , skipEmpty
+    , LinesWithComments(..)
 #endif
     ) where
 
 import Prelude hiding (lines)
-import Database.Persist.Types
+
+import qualified Data.List.NonEmpty as NEL
+import Data.List.NonEmpty (NonEmpty(..))
+import Control.Arrow ((&&&))
+import Control.Monad (msum, mplus)
 import Data.Char
-import Data.Maybe (mapMaybe, fromMaybe, maybeToList)
+import Data.List (find, foldl')
+import qualified Data.Map as M
+import Data.Maybe (mapMaybe, fromMaybe, maybeToList, listToMaybe)
+import Data.Monoid (mappend)
 import Data.Text (Text)
 import qualified Data.Text as T
-import Control.Arrow ((&&&))
-import qualified Data.Map as M
-import Data.List (find, foldl')
-import Data.Monoid (mappend)
-import Control.Monad (msum, mplus)
+import Database.Persist.Types
 
 data ParseState a = PSDone | PSFail String | PSSuccess a Text deriving Show
 
@@ -109,21 +355,26 @@ lowerCaseSettings = defaultPersistSettings
 
 -- | Parses a quasi-quoted syntax into a list of entity definitions.
 parse :: PersistSettings -> Text -> [EntityDef]
-parse ps = parseLines ps
-      . removeSpaces
-      . filter (not . empty)
-      . map tokenize
-      . T.lines
+parse ps = parseLines ps . preparse
+
+preparse :: Text -> [Line]
+preparse =
+    removeSpaces
+        . filter (not . empty)
+        . map tokenize
+        . T.lines
 
 -- | A token used by the parser.
 data Token = Spaces !Int   -- ^ @Spaces n@ are @n@ consecutive spaces.
            | Token Text    -- ^ @Token tok@ is token @tok@ already unquoted.
+           | DocComment Text -- ^ @DocComment@ is a documentation comment, unmodified.
   deriving (Show, Eq)
 
 -- | Tokenize a string.
 tokenize :: Text -> [Token]
 tokenize t
     | T.null t = []
+    | "-- | " `T.isPrefixOf` t = [DocComment t]
     | "--" `T.isPrefixOf` t = [] -- Comment until the end of the line.
     | "#" `T.isPrefixOf` t = [] -- Also comment to the end of the line, needed for a CPP bug (#110)
     | T.head t == '"' = quotes (T.tail t) id
@@ -181,9 +432,22 @@ empty _          = False
 
 -- | A line.  We don't care about spaces in the middle of the
 -- line.  Also, we don't care about the ammount of indentation.
-data Line = Line { lineIndent :: Int
-                 , tokens     :: [Text]
-                 }
+data Line' f
+    = Line
+    { lineIndent   :: Int
+    , tokens       :: f Text
+    }
+
+deriving instance Show (f Text) => Show (Line' f)
+deriving instance Eq (f Text) => Eq (Line' f)
+
+mapLine :: (forall x. f x -> g x) -> Line' f -> Line' g
+mapLine k (Line i t) = Line i (k t)
+
+traverseLine :: Functor t => (forall x. f x -> t (g x)) -> Line' f -> t (Line' g)
+traverseLine k (Line i xs) = Line i <$> k xs
+
+type Line = Line' []
 
 -- | Remove leading spaces and remove spaces in the middle of the
 -- tokens.
@@ -197,6 +461,7 @@ removeSpaces =
     toLine' i = Line i . mapMaybe fromToken
 
     fromToken (Token t) = Just t
+    fromToken (DocComment t) = Just t
     fromToken Spaces{}  = Nothing
 
 -- | Divide lines into blocks and make entity definitions.
@@ -204,11 +469,87 @@ parseLines :: PersistSettings -> [Line] -> [EntityDef]
 parseLines ps lines =
     fixForeignKeysAll $ toEnts lines
   where
-    toEnts (Line indent (name:entattribs) : rest) =
-        let (x, y) = span ((> indent) . lineIndent) rest
-        in  mkEntityDef ps name entattribs x : toEnts y
-    toEnts (Line _ []:rest) = toEnts rest
-    toEnts [] = []
+    toEnts :: [Line] -> [UnboundEntityDef]
+    toEnts =
+        map mk
+        . associateLines
+        . skipEmpty
+    mk :: LinesWithComments -> UnboundEntityDef
+    mk lwc =
+        let Line _ (name :| entAttribs) :| rest = lwcLines lwc
+         in setComments (lwcComments lwc) $ mkEntityDef ps name entAttribs (map (mapLine NEL.toList) rest)
+
+isComment :: Text -> Maybe Text
+isComment xs =
+    T.stripPrefix "-- | " xs
+
+data LinesWithComments = LinesWithComments
+    { lwcLines :: NonEmpty (Line' NonEmpty)
+    , lwcComments :: [Text]
+    } deriving (Eq, Show)
+
+-- TODO: drop this and use <> when 8.2 isn't supported anymore so the
+-- monoid/semigroup nonsense isn't annoying
+appendLwc :: LinesWithComments -> LinesWithComments -> LinesWithComments
+appendLwc a b =
+    LinesWithComments (foldr NEL.cons (lwcLines b) (lwcLines a)) (lwcComments a `mappend` lwcComments b)
+
+newLine :: Line' NonEmpty -> LinesWithComments
+newLine l = LinesWithComments (pure l) []
+
+firstLine :: LinesWithComments -> Line' NonEmpty
+firstLine = NEL.head . lwcLines
+
+consLine :: Line' NonEmpty -> LinesWithComments -> LinesWithComments
+consLine l lwc = lwc { lwcLines = NEL.cons l (lwcLines lwc) }
+
+consComment :: Text -> LinesWithComments -> LinesWithComments
+consComment l lwc = lwc { lwcComments = l : lwcComments lwc }
+
+associateLines :: [Line' NonEmpty] -> [LinesWithComments]
+associateLines lines =
+    foldr combine [] $
+    foldr toLinesWithComments [] lines
+  where
+    toLinesWithComments line linesWithComments =
+        case linesWithComments of
+            [] ->
+                [newLine line]
+            (lwc : lwcs) ->
+                case isComment (NEL.head (tokens line)) of
+                    Just comment
+                        | lineIndent line == lowestIndent ->
+                        consComment comment lwc : lwcs
+                    _ ->
+                        if lineIndent line <= lineIndent (firstLine lwc)
+                        then
+                            consLine line lwc : lwcs
+                        else
+                            newLine line : lwc : lwcs
+
+    lowestIndent = minimum . fmap lineIndent $ lines
+    combine :: LinesWithComments -> [LinesWithComments] -> [LinesWithComments]
+    combine lwc [] =
+        [lwc]
+    combine lwc (lwc' : lwcs) =
+        let minIndent = minimumIndentOf lwc
+            otherIndent = minimumIndentOf lwc'
+         in
+            if minIndent < otherIndent then
+                appendLwc lwc lwc' : lwcs
+            else
+                lwc : lwc' : lwcs
+
+
+    minimumIndentOf = minimum . fmap lineIndent . lwcLines
+
+skipEmpty :: [Line' []] -> [Line' NonEmpty]
+skipEmpty = mapMaybe (traverseLine NEL.nonEmpty)
+
+setComments :: [Text] -> UnboundEntityDef -> UnboundEntityDef
+setComments [] = id
+setComments comments =
+    overUnboundEntityDef (\ed -> ed { entityComments = Just (T.unlines comments) })
 
 fixForeignKeysAll :: [UnboundEntityDef] -> [EntityDef]
 fixForeignKeysAll unEnts = map fixForeignKeys unEnts
@@ -223,24 +564,41 @@ fixForeignKeysAll unEnts = map fixForeignKeys unEnts
     -- check the count and the sqltypes match and update the foreignFields with the names of the primary columns
     fixForeignKey :: EntityDef -> UnboundForeignDef -> ForeignDef
     fixForeignKey ent (UnboundForeignDef foreignFieldTexts fdef) =
-        case M.lookup (foreignRefTableHaskell fdef) entLookup of
-          Just pent -> case entityPrimary pent of
-             Just pdef ->
-                 if length foreignFieldTexts /= length (compositeFields pdef)
-                   then lengthError pdef
-                   else let fds_ffs = zipWith (toForeignFields pent)
-                                foreignFieldTexts
-                                (compositeFields pdef)
-                        in  fdef { foreignFields = map snd fds_ffs
-                                 , foreignNullable = setNull $ map fst fds_ffs
-                                 }
-             Nothing ->
-                 error $ "no explicit primary key fdef="++show fdef++ " ent="++show ent
-          Nothing ->
-             error $ "could not find table " ++ show (foreignRefTableHaskell fdef)
-               ++ " fdef=" ++ show fdef ++ " allnames="
-               ++ show (map (unHaskellName . entityHaskell . unboundEntityDef) unEnts)
-               ++ "\n\nents=" ++ show ents
+        let pentError =
+                error $ "could not find table " ++ show (foreignRefTableHaskell fdef)
+                ++ " fdef=" ++ show fdef ++ " allnames="
+                ++ show (map (unHaskellName . entityHaskell . unboundEntityDef) unEnts)
+                ++ "\n\nents=" ++ show ents
+            pent =
+                fromMaybe pentError $ M.lookup (foreignRefTableHaskell fdef) entLookup
+         in
+            case entityPrimary pent of
+                Just pdef ->
+                    if length foreignFieldTexts /= length (compositeFields pdef)
+                    then
+                        lengthError pdef
+                    else
+                        let
+                            fds_ffs =
+                                zipWith (toForeignFields pent)
+                                    foreignFieldTexts
+                                    (compositeFields pdef)
+                            dbname =
+                                unDBName (entityDB pent)
+                            oldDbName =
+                                unDBName (foreignRefTableDBName fdef)
+                         in fdef
+                            { foreignFields = map snd fds_ffs
+                            , foreignNullable = setNull $ map fst fds_ffs
+                            , foreignRefTableDBName =
+                                DBName dbname
+                            , foreignConstraintNameDBName =
+                                DBName
+                                . T.replace oldDbName dbname . unDBName
+                                $ foreignConstraintNameDBName fdef
+                            }
+                Nothing ->
+                    error $ "no explicit primary key fdef="++show fdef++ " ent="++show ent
       where
         setNull :: [FieldDef] -> Bool
         setNull [] = error "setNull: impossible!"
@@ -282,9 +640,14 @@ data UnboundEntityDef = UnboundEntityDef
                         , unboundEntityDef :: EntityDef
                         }
 
+overUnboundEntityDef
+    :: (EntityDef -> EntityDef) -> UnboundEntityDef -> UnboundEntityDef
+overUnboundEntityDef f ubed =
+    ubed { unboundEntityDef = f (unboundEntityDef ubed) }
 
 lookupKeyVal :: Text -> [Text] -> Maybe Text
 lookupKeyVal key = lookupPrefix $ key `mappend` "="
+
 lookupPrefix :: Text -> [Text] -> Maybe Text
 lookupPrefix prefix = msum . map (T.stripPrefix prefix)
 
@@ -297,20 +660,23 @@ mkEntityDef :: PersistSettings
 mkEntityDef ps name entattribs lines =
   UnboundEntityDef foreigns $
     EntityDef
-        entName
-        (DBName $ getDbName ps name' entattribs)
+        { entityHaskell = entName
+        , entityDB = DBName $ getDbName ps name' entattribs
         -- idField is the user-specified Id
         -- otherwise useAutoIdField
         -- but, adjust it if the user specified a Primary
-        (setComposite primaryComposite $ fromMaybe autoIdField idField)
-        entattribs
-        cols
-        uniqs
-        []
-        derives
-        extras
-        isSum
+        , entityId = (setComposite primaryComposite $ fromMaybe autoIdField idField)
+        , entityAttrs = entattribs
+        , entityFields = cols
+        , entityUniques = uniqs
+        , entityForeigns = []
+        , entityDerives = derives
+        , entityExtra = extras
+        , entitySum = isSum
+        , entityComments = comments
+        }
   where
+    comments = Nothing
     entName = HaskellName name'
     (isSum, name') =
         case T.uncons name of
@@ -330,7 +696,18 @@ mkEntityDef ps name entattribs lines =
     derives = concat $ mapMaybe takeDerives attribs
 
     cols :: [FieldDef]
-    cols = mapMaybe (takeColsEx ps) attribs
+    cols = reverse . fst . foldr k ([], []) $ reverse attribs
+    k x (!acc, !comments) =
+        case isComment =<< listToMaybe x of
+            Just comment ->
+                (acc, comment : comments)
+            Nothing ->
+                ( maybe id (:) (setFieldComments comments <$> takeColsEx ps x) acc
+                , []
+                )
+    setFieldComments [] x = x
+    setFieldComments xs fld =
+        fld { fieldComments = Just (T.unlines xs) }
 
     autoIdField = mkAutoIdField ps entName (DBName `fmap` idName) idSqlType
     idSqlType = maybe SqlInt64 (const $ SqlOther "Primary Key") primaryComposite
@@ -358,6 +735,7 @@ mkAutoIdField ps entName idName idSqlType = FieldDef
       , fieldReference = ForeignRef entName  defaultReferenceTypeCon
       , fieldAttrs = []
       , fieldStrict = True
+      , fieldComments = Nothing
       }
 
 defaultReferenceTypeCon :: FieldType
@@ -379,9 +757,15 @@ splitExtras (Line _ ts:rest) =
      in (ts:x, y)
 
 takeColsEx :: PersistSettings -> [Text] -> Maybe FieldDef
-takeColsEx = takeCols (\ft perr -> error $ "Invalid field type " ++ show ft ++ " " ++ perr)
+takeColsEx =
+    takeCols
+        (\ft perr -> error $ "Invalid field type " ++ show ft ++ " " ++ perr)
 
-takeCols :: (Text -> String -> Maybe FieldDef) -> PersistSettings -> [Text] -> Maybe FieldDef
+takeCols
+    :: (Text -> String -> Maybe FieldDef)
+    -> PersistSettings
+    -> [Text]
+    -> Maybe FieldDef
 takeCols _ _ ("deriving":_) = Nothing
 takeCols onErr ps (n':typ:rest)
     | not (T.null n) && isLower (T.head n) =
@@ -395,6 +779,7 @@ takeCols onErr ps (n':typ:rest)
                 , fieldAttrs = rest
                 , fieldStrict = fromMaybe (psStrictFields ps) mstrict
                 , fieldReference = NoReference
+                , fieldComments = Nothing
                 }
   where
     (mstrict, n)
@@ -526,13 +911,21 @@ takeForeign :: PersistSettings
 takeForeign ps tableName _defs (refTableName:n:rest)
     | not (T.null n) && isLower (T.head n)
         = UnboundForeignDef fields $ ForeignDef
-            (HaskellName refTableName)
-            (DBName $ psToDBName ps refTableName)
-            (HaskellName n)
-            (DBName $ psToDBName ps (tableName `T.append` n))
-            []
-            attrs
-            False
+            { foreignRefTableHaskell =
+                HaskellName refTableName
+            , foreignRefTableDBName =
+                DBName $ psToDBName ps refTableName
+            , foreignConstraintNameHaskell =
+                HaskellName n
+            , foreignConstraintNameDBName =
+                DBName $ psToDBName ps (tableName `T.append` n)
+            , foreignFields =
+                []
+            , foreignAttrs =
+                attrs
+            , foreignNullable =
+                False
+            }
   where
     (fields,attrs) = break ("!" `T.isPrefixOf`) rest
 
